#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QPalette>
#include <QQmlContext>
#include <QDebug>

class MyStyle : public QObject
{
    Q_OBJECT
public:
    MyStyle() : QObject(nullptr) {
        toLight();
    }

    Q_INVOKABLE bool isDark(const QColor &color) const {
        QColor rgb_color = color.toRgb();
        float luminance = 0.299 * rgb_color.redF() + 0.587 * rgb_color.greenF() + 0.114 * rgb_color.blueF();

        if (qRound(luminance * 255) > 191) {
            return false;
        }
        return true;
    }

    Q_INVOKABLE QColor sliderBackground(const QColor &base) const
    {
        return isDark(base) ? "#303030" : "#e0e0e0";
    }

    Q_INVOKABLE void toLight() {
        QPalette pa = qGuiApp->palette();
        pa.setColor(QPalette::Window, Qt::white);
        pa.setColor(QPalette::WindowText, Qt::black);
        qGuiApp->setPalette(pa);
    }

    Q_INVOKABLE void toDark() {
        QPalette pa = qGuiApp->palette();
        pa.setColor(QPalette::Window, Qt::black);
        pa.setColor(QPalette::WindowText, Qt::white);
        qGuiApp->setPalette(pa);
    }
};

int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);

    QPalette palette = app.palette();
    palette.setColor(QPalette::Midlight, Qt::red);
    app.setPalette(palette);

    QQmlApplicationEngine engine;
    engine.rootContext()->setContextProperty("mystyle", new MyStyle());
    engine.addImportPath(":/");
    const QUrl url(QStringLiteral("qrc:/main.qml"));
    QObject::connect(&engine, &QQmlApplicationEngine::objectCreated,
                     &app, [url](QObject *obj, const QUrl &objUrl) {
        if (!obj && url == objUrl)
            QCoreApplication::exit(-1);
    }, Qt::QueuedConnection);
    engine.load(url);

    return app.exec();
}

#include "main.moc"
